@@ -33,6 +33,10 @@ module Sus
 			duration
 		end
 		
+		def ms
+			duration * 1000.0
+		end
+		
 		def to_s
 			duration = self.duration
 			
@@ -45,6 +49,10 @@ module Sus
 			end
 		end
 		
+		def reset!(duration = 0.0)
+			@duration = duration
+		end
+		
 		def start!
 			@start_time = Process.clock_gettime(Process::CLOCK_MONOTONIC)
 		end
